import { any, hasIn, is, path, pipe, split } from 'ramda';
var INDENTATION = 12;
var prop = pipe(split(/[[\].]/), path);
export var isListElement = function (val) {
    return is(Object, val) && !prop('key')(val);
};
export var isDictionaryElement = function (value) {
    return is(Object, value) &&
        hasIn('key', value) &&
        (is(String, prop('key')(value)) || is(Number, prop('key')(value)));
};
export var isMetaList = function (value) {
    return is(Array, value) && any(isListElement, value);
};
export var isMetaDictionary = function (value) {
    return is(Array, value) && any(isDictionaryElement, value);
};
var stringify = function (space, val, padding) {
    var indentation = ' '.repeat(padding);
    return JSON.stringify(val, null, space)
        .replace(/\\n/g, '\n')
        .replace(/}$/, indentation + '}')
        .replace(/]$/, indentation + ']');
};
export function normalizeIssueVariable(keyValue, value, padding, indentation, withoutQuotes) {
    var _a;
    if (indentation === void 0) { indentation = INDENTATION; }
    if (is(String, value)) {
        var elementPadding_1 = ' '.repeat((withoutQuotes ? 3 : 5) + padding + keyValue.length);
        var formattedValue = value
            .split(/\n/)
            .map(function (item, index) { return (index === 0 ? item : "" + elementPadding_1 + item); })
            .join('\n');
        return "\"" + formattedValue + "\"";
    }
    if (isMetaList(value) || isMetaDictionary(value)) {
        var items = value.filter(Boolean).map(function (item) {
            if (is(String, item)) {
                return item;
            }
            var key = item.key, value = item.value;
            return key ? key + ": " + value : value;
        });
        return stringify(indentation, items, padding);
    }
    if (isDictionaryElement(value)) {
        return stringify(keyValue.length, (_a = {}, _a[value.key] = value.value, _a), padding);
    }
    if (isListElement(value)) {
        if (is(Array, value.value)) {
            return stringify(indentation, value.value, padding);
        }
        return String(value.value);
    }
    return String(value);
}
